const path = require("path")
const util = require("../util")
const webpack = require("webpack")
const VueLoaderPlugin = require('vue-loader/lib/plugin');
const SpeedMeasurePlugin = require("speed-measure-webpack-plugin");
const merge = require('webpack-merge')
// const threadLoader = require('thread-loader');
// threadLoader.warmup({
//   poolTimeout: 2000
// }, ['vue-loader', 'postcss-loader', 'babel-loader', 'eslint-loader']);

const TerserPlugin = require('terser-webpack-plugin');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');

const MiniCssExtractPlugin = require('mini-css-extract-plugin')

const OptimizeCSSPlugin = require('optimize-css-assets-webpack-plugin')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const devModuleRules = require("./webpack.config.dev").module.rules
const { userConfig, templates, webpackConfig } = require("./webpack.config.base")
const context = webpackConfig.context
let publicPath = userConfig.publicPath
const smp = new SpeedMeasurePlugin();

// 默认自己公司的配置
if (!publicPath) {
  let publicDomain = '//static.sunyur.com/'
  publicPath = publicDomain + path.basename(context).replace('style-', '') + '/'
}

const userConfigWebpack = userConfig && userConfig.webpackConfig && userConfig.webpackConfig.prod || {}
// 异步模块打包 @todo 临时解决方案
const basePath = templates[0].basePath
let config = merge(webpackConfig, {
  output: Object.assign({}, webpackConfig.output, {
    // 输出的文件名 hash统一生成,chunkhash变化生成
    filename: '[name].js',
    // chunkFilename: basePath + '/[id].[chunkhash:8].js'
    chunkFilename: userConfig.chunkhash ? basePath + '/[name].[chunkhash:' + userConfig.chunkhash + '].js' :basePath + '/[name].[chunkhash:8].js'
  }),
  module: {
    rules: [
      
      // vue
      // {
      //   test: /\.vue$/,
      //   use:[
      //     'thread-loader',
      //     {
      //       loader: 'vue-loader',
      //       options: {
      //         loaders: util.extractVueStlye(),
      //         preserveWhitespace: false
      //       }}
      //   ]
      // },
      {
        test: /\.vue$/,
        use: [
          // "thread-loader",
          'vue-loader'
        ]
        // loader:
      },
      
      ...util.stlyeRules(userConfig["devtool"], false, userConfig)
    ]
      // style
      
    // ) : devModuleRules
  },
  
  performance: {
    maxEntrypointSize: 1000000,
    maxAssetSize: 1000000
  },
  mode: 'production',
  plugins: [
    // 配置环境
    new webpack.DefinePlugin({
      'process.env': {
        NODE_ENV: '"production"'
      },
      IS_DEV: JSON.stringify(false)
    }),
    new VueLoaderPlugin(),
    // 对css压缩
    new webpack.LoaderOptionsPlugin({
      minimize: true
    }),

    // 当模块内容没有修改时，保持module.id不变
    new webpack.HashedModuleIdsPlugin()

  ],
  // 对js进行压缩混淆 webpack 4
  optimization: {
    nodeEnv: 'production',
    splitChunks: {
      automaticNameDelimiter: '_'
    },
    minimize: true,
    concatenateModules: true,
    minimizer: [
        // new UglifyJsPlugin({
        //   include: /\.js$/,
        //   parallel: true,
        //   cache: true,
        //   uglifyOptions: {
        //     compress: {
        //       // warnings: true, // 在UglifyJs删除没有用到的代码时不输出警告
        //       // drop_console: true, // 删除所有的 `console` 语句，可以兼容ie浏览器
        //       collapse_vars: true, // 内嵌定义了但是只用到一次的变量
        //       reduce_vars: true // 提取出出现多次但是没有定义成变量去引用的静态值
        //     }
        //   }
        // })
        new TerserPlugin({
          include: /\.js$/,
          parallel: true,
          cache: true,
          terserOptions: {
            ecma: undefined,
            warnings: false,
            parse: {},
            compress: {
              drop_console: true
            },
            mangle: true, // Note `mangle.properties` is `false` by default.
            module: false,
            output: null,
            toplevel: false,
            nameCache: null,
            ie8: false,
            keep_classnames: undefined,
            keep_fnames: false,
            safari10: false,
          }
        })
      // new TerserPlugin({
      //   cache: true,
      //   parallel: true,
      //   sourceMap: true
      // })
    ]
  },
  // 上线设置source-map会影响加载速度,但便于线上bug查找
  devtool: userConfig["devtool"],

  //编译期间一旦出错就停止编译
  bail: true,
  stats: 'errors-only',
}, userConfigWebpack)

// webpack3.0  模块作用域提升
if (webpack.optimize.ModuleConcatenationPlugin) {
  config.plugins.push(new webpack.optimize.ModuleConcatenationPlugin())
}

// 提取文件的公共部分
if (userConfig && userConfig.vendor) {
  var filename = userConfig.chunkhash ? 'common.[chunkhash:' + userConfig.chunkhash + '].js' : 'common.js'
  templates.forEach((tpl, index) => {
    const common = tpl.basePath + '/common'
    const templateName = path.join(tpl.basePath, 'index.html')
    config.plugins.push(new HtmlWebpackPlugin({
      filename: templateName,
      template: tpl.template,
      chunks: [common, tpl.chunk],
      minify: {
        keepClosingSlash: true,
        removeAttributeQuotes: false,
        quoteCharacter: '"',
      },
      xhtml: false,
      inject: true
    }))
  })
  
} else{
  templates.forEach((tpl) => {
    const filename = path.join(tpl.basePath, 'index.html')
    config.plugins.push(new HtmlWebpackPlugin({
      filename,
      template: tpl.template,
      chunks: [tpl.chunk],
      inject: true
    }))
  })
}

if (userConfig.extractCSS) {
  config.plugins.push(
    // 提取css,生成对应对文件
    new MiniCssExtractPlugin({
      filename: userConfig.chunkhash ? '[name].[chunkhash:' + userConfig.chunkhash + '].css' : '[name].css',
      chunkFilename: userConfig.chunkhash ? basePath + '/[name].[chunkhash:' + userConfig.chunkhash + '].css' :basePath + '/[name].[chunkhash:8].css'
      // filename: '/[name].css',
    }),
    // 提取的css可能存在重复的样式，去掉重复的
    new OptimizeCSSPlugin({
      cssProcessorOptions: {
        safe: true
      }
    })
  )
}


if (userConfig.bundleAnalyzerReport) {
  var BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin
  config.plugins.push(new BundleAnalyzerPlugin())
}
// console.log(config.rules)
// module.exports = smp.wrap(config)
module.exports = config
