const webpackConfig = require('./webpack.config.dev')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const webpack = require('webpack')
const util = require('../util')
const $path = require('path')
const FriendlyErrorsPlugin = require('friendly-errors-webpack-plugin')
const VueLoaderPlugin = require('vue-loader/lib/plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const merge = require('webpack-merge')

const chalk = require('chalk')
const userConfig = require('./webpack.config.base').userConfig
const { templates } = require('./webpack.config.base')

const context = webpackConfig.context
const webpackClientPath = $path.join(__dirname, 'webpack.client.js')
const userConfigWebpack = userConfig && userConfig.webpackConfig && userConfig.webpackConfig.start || {}

// 对每一个入口文件添加 hot-reload 依赖
Object.keys(webpackConfig.entry).forEach(function (name) {
  webpackConfig.entry[name] = [webpackClientPath].concat(webpackConfig.entry[name])
})

// html插件
const htmlPlugin = []
templates.forEach((tpl, index) => {
  if (templates.length === 1) {
    htmlPlugin.push(new HtmlWebpackPlugin({
      template: tpl.template
    }))
  } else {
    if (!util.isFile($path.resolve(context, tpl.template))) {
      console.log(chalk.bold.red('\n[syue][start]' + tpl.template + '模板路径没找到\n'));
      process.exit(1)
    }
    console.log(`\n访问文件的地址：/${tpl.filename}\n`)
    htmlPlugin.push(new HtmlWebpackPlugin({
      filename: tpl.filename,
      template: tpl.template,
      chunks: [tpl.chunk]
    }))

    // 添加一个入口文件
    if (index === 0) {
      htmlPlugin.push(new HtmlWebpackPlugin({
        pages: templates,
        template: $path.join(__dirname, 'index.hbs'),
        inject: true
      }))
    }
  }
})

var config = merge(webpackConfig, {
  output: {
    publicPath: '/'
  },
  mode: 'development',
  stats:{
    modules: false,
    children: false,
    chunks: false,
    chunkModules: false,
    moduleTrace: false,
    warningsFilter: [
      'mini-css-extract-plugin'
    ]
  },
  module: {
    rules: [
      // vue
      {
        test: /\.vue$/,
        loader: 'vue-loader',
        options: {
          preserveWhitespace: false
        }
      },
      // style
      ...util.stlyeRules(false, true, userConfig)
    ]
  },
  plugins: [
    new webpack.DefinePlugin({
      'process.env': {
        NODE_ENV: '"development"'
      },
      IS_DEV: JSON.stringify(true)
    }),
    new MiniCssExtractPlugin({
      filename: userConfig.chunkhash ? '[name].[chunkhash:' + userConfig.chunkhash + '].css' : '[name].css',
      allChunks: true
    }),
    new VueLoaderPlugin(),
    new webpack.HotModuleReplacementPlugin(),
    // //引入模版文件
    // new HtmlWebpackPlugin({
    //   template: userConfig.index
    // }),
    new FriendlyErrorsPlugin()
  ].concat(htmlPlugin)
}, userConfigWebpack)
module.exports = config
