const program = require('commander')
const chalk = require('chalk')
const _ = require('lodash')
const path = require('path')
const fs = require('fs-extra')
const util = require('../util')
const packageConfig = require('../package.json')
const packageTemplate = require('../template/vue/package-template.json')
// 项目名
let projectName

program
    .arguments('<project-directory>')
    .option('-c, --cover', '如果工程存在，就覆盖掉')
    .action(function (name) {
        projectName = name
    })
    .parse(process.argv)

// 如果没有输入项目名称
if (typeof projectName === 'undefined') {
    console.error(chalk.bold.red('Please specify the project directory(请指定项目目录):'))
    console.log('Run ' + packageConfig._name + ' --help to see all options.\n')
    process.exit(1)
}

// 获取项目路径
var projectPath = path.resolve(projectName)
// 如果项目目录存在 并且不允许覆盖 提示错误
if(util.exists(projectPath) && !program.cover) {
    console.error(chalk.bold.red('The project directory has existed(项目目录已经存在)'+ '\n'))
    process.exit(1)
}

// 创建项目
createProject(projectPath, projectName)

function createProject(projectPath, projectName) {
    let templatePath = '../template/vue'

    // 如果没有这个文件夹就创建一个
    if (util.exists(projectPath)) {
        fs.mkdirsSync(projectPath)
    }
    // 修改 package
    packageTemplate.name = projectName
    packageTemplate.devDependencies[packageConfig.name] = packageConfig.version

    fs.copySync(path.resolve(__dirname, templatePath), projectPath)
    fs.writeFileSync(
        path.join(projectPath, 'package.json'),
        JSON.stringify(packageTemplate, null, 2)
    )
    console.log('Success! Created ' + projectName + ' at ' + projectPath)
}