process.env.NODE_ENV = 'development'
process.traceDeprecation = true
const $path = require('path')
const fs = require('fs')
const https = require('https')
const express = require('express')
const util = require('../util')
const proxyMiddleware = require('http-proxy-middleware')
let { userConfig, templates } = require('../config/webpack.config.base')
let serverConfig = require('../config/server.config')
let webpackConfig = require('../config/webpack.config.start')
let opn = require('opn')
let webpack = require('webpack')
var os = require('os')

serverConfig = Object.assign({}, serverConfig, userConfig)

const app = express()
const compiler = webpack(webpackConfig)

let devMiddleware = require('webpack-dev-middleware')(compiler, {
  publicPath: webpackConfig.output.publicPath,
  noInfo: false,
  quiet: true,
  logLevel: 'warn',
})

const protocol = serverConfig.protocol === 'https' ? 'https' : 'http'
const url = protocol + '://127.0.0.1:' + serverConfig.port
let urlStr = url
const ip = getIp()
if (ip) {
  urlStr += '  ' + protocol + '://' + ip + ':' + serverConfig.port
}
devMiddleware.waitUntilValid(function () {
  console.log(`> Listening at ${urlStr}\n`)
})

var hotMiddleware = require('webpack-hot-middleware')(compiler, {
  log: false,
  heartbeat: 2000
})

// force page reload when html-webpack-plugin template changes
// compiler.hooks.compile.tap('MyPlugin', params => {
//   console.log('以同步方式触及 compile 钩子。');
// })
compiler.hooks.compile.tap('compilation', function (compilation) {
  compiler.hooks.compile.tap('html-webpack-plugin-after-emit', function (data, cb) {
    hotMiddleware.publish({ action: 'reload' })
    cb && cb()
  })
})

// proxy api requests
if (serverConfig.proxyTable) {
  Object.keys(serverConfig.proxyTable).forEach(function (context) {
    var options = serverConfig.proxyTable[context]
    if (typeof options === 'string') {
      options = { target: options }
    }
    app.use(proxyMiddleware(options.filter || context, options))
  })
}

// mock api requests
if (serverConfig.mockTable) {
  Object.keys(serverConfig.mockTable).forEach(function (context) {
    var options = serverConfig.mockTable[context]
    var mockFile = $path.resolve(webpackConfig.context, options)
    if (util.isFile(mockFile)) {
      app.use(context, require(mockFile))
    } else {
      console.log('[syue][start]mockTable:', context, options, '文件不存在')
    }
  })
}

// handle fallback for HTML5 history API
app.use(require('connect-history-api-fallback')())

app.use(devMiddleware)
app.use(hotMiddleware)

// 静态资源
app.use('/static', express.static('static'))
// 每个页面的资源路径
templates.forEach(tpl => {
  let staticPath = tpl.filePath.substring(0, tpl.filePath.indexOf('src'))
  staticPath = $path.join(staticPath, 'assets')
  console.log('[syue][start]staticPath:', staticPath)
  app.use('/assets', express.static(staticPath))
})

function callback (err) {
  if (err) {
    console.log('[error]启动失败：', err)
    return
  }
  console.log(url)
  if (serverConfig.autoOpenBrowser) {
    opn(url).then(()=> {
      // console.log('')
    }).catch((err)=>{
      console.log('[error]启动失败：', err)
    })
  }
}

if (serverConfig.protocol === 'https') {
  const privateKey = fs.readFileSync($path.join(__dirname, 'server.key'), 'utf8')
  const certificate = fs.readFileSync($path.join(__dirname, 'server.crt'), 'utf8')
  // 创建https服务
  https.createServer({ key: privateKey, cert: certificate }, app).listen(serverConfig.port, callback)
} else {
  app.listen(serverConfig.port, callback)
}

function getIp () {
  var interfaces = os.networkInterfaces()
  var en = interfaces.en0
  var ipv4
  if (en) {
    en.forEach(function (details) {
      if (details.family === 'IPv4') {
        ipv4 = details.address
      }
    })
  }
  return ipv4
}